"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseAssertion = void 0;
const node_process_1 = __importDefault(require("process"));
const node_assert_1 = __importDefault(require("assert"));
const node_os_1 = require("os");
const format_js_1 = require("./format.js");
const hr_js_1 = require("./hr.js");
const findFile = (error) => {
    var _a;
    const stackLines = ((_a = error.stack) === null || _a === void 0 ? void 0 : _a.split(node_os_1.EOL)) || [];
    let file = '';
    const basePath = 'poku/lib/';
    for (const line of stackLines) {
        if (!line.includes(basePath)) {
            const match = line.match(/at\s(\/.+|file:\/\/\/.+)/i);
            if (match && match[1]) {
                file = match[1];
                break;
            }
        }
    }
    return file;
};
const parseAssertion = (cb, options) => {
    try {
        cb();
    }
    catch (error) {
        if (error instanceof node_assert_1.default.AssertionError) {
            const { code, actual, expected, operator } = error;
            const file = findFile(error);
            (0, hr_js_1.hr)();
            if (typeof options.message === 'string')
                console.log(format_js_1.format.bold(options.message), node_os_1.EOL);
            else if (options.message instanceof Error)
                console.log(format_js_1.format.bold(options.message.message), node_os_1.EOL);
            else if (typeof options.defaultMessage === 'string')
                console.log(options.defaultMessage, node_os_1.EOL);
            console.log(format_js_1.format.dim('Code:    '), format_js_1.format.bold(format_js_1.format.fail(code)));
            file && console.log(format_js_1.format.dim('File:    '), file);
            console.log(format_js_1.format.dim('Operator:'), operator);
            (0, hr_js_1.hr)();
            if (!(options === null || options === void 0 ? void 0 : options.hideDiff)) {
                console.log(format_js_1.format.dim(`${(options === null || options === void 0 ? void 0 : options.actual) || 'Actual'}:`));
                console.log(format_js_1.format.bold(typeof actual === 'function' || actual instanceof RegExp
                    ? String(actual)
                    : format_js_1.format.fail(JSON.stringify(actual))));
                console.log(`${node_os_1.EOL}${format_js_1.format.dim(`${(options === null || options === void 0 ? void 0 : options.expected) || 'Expected'}:`)}`);
                console.log(format_js_1.format.bold(typeof expected === 'function' || expected instanceof RegExp
                    ? String(expected)
                    : format_js_1.format.success(JSON.stringify(expected))));
                (0, hr_js_1.hr)();
            }
            if (options.throw) {
                console.log(error);
                (0, hr_js_1.hr)();
            }
            node_process_1.default.exit(1);
        }
        // Non-assertion errors
        throw error;
    }
};
exports.parseAssertion = parseAssertion;
